@@ -26,7 +26,7 @@ class Main extends MY_Controller {
 
 		// Try to guess what the user would like to do.
 		$id = $this->uri->segment(1);
-		if (strpos($id, "m-") === 0 && $this->mmultipaste->id_exists($id)) {
+		if (isset($id) && strpos($id, "m-") === 0 && $this->mmultipaste->id_exists($id)) {
 			$this->_download();
 		} elseif ($id != "file" && $this->mfile->id_exists($id)) {
 			$this->_download();
@@ -98,7 +98,7 @@ class Main extends MY_Controller {
 	{
 		session_write_close();
 		$id = $this->uri->segment(1);
-		$lexer = urldecode($this->uri->segment(2));
+		$lexer = urldecode($this->uri->segment(2) ?? '');
 
 		$is_multipaste = false;
 		if ($this->mmultipaste->id_exists($id)) {
@@ -468,6 +468,7 @@ class Main extends MY_Controller {
 			if ($filedata !== false) {
 				$pygments = new \libraries\Pygments($this->mfile->file($filedata["data_id"]), $filedata["mimetype"], $filedata["filename"]);
 				if ($pygments->can_highlight()) {
+					$this->data["textarea_filename"] = $filedata["filename"];
 					$this->data["textarea_content"] = file_get_contents($this->mfile->file($filedata["data_id"]));
 				}
 			}
